@@ -306,7 +306,7 @@ void EST_BackoffNgrammarState::print_freqs(ostream &os,
     // not right - just print out, then recurse through children
     // change to use 'backoff_traverse'
     
-    int k;
+    EST_Litem *k;
     double freq;
     EST_String name;
     for (k=p_pdf.item_start();
@@ -373,7 +373,7 @@ void EST_BackoffNgrammarState::zap()
 {
 
     // recursively delete this state and all its children
-    int k;
+    EST_Litem *k;
     double freq;
     EST_String name;
     for (k=p_pdf.item_start();
@@ -456,7 +456,8 @@ bool EST_BackoffNgrammarState::set_backoff_weight(const EST_StrVector &words, co
 
 void EST_BackoffNgrammarState::frequency_of_frequencies(EST_DVector &ff)
 {
-    int k,max=ff.n();
+    int max=ff.n();
+    EST_Litem *k;
     double freq;
     EST_String name;
     for (k=p_pdf.item_start();
@@ -1585,7 +1586,7 @@ void EST_Ngrammar::prune_backoff_representation(EST_BackoffNgrammarState *start_
     // remove any branches with zero frequency count
     
     // find children of this state with zero freq and zap them
-    int k;
+    EST_Litem *k;
     double freq;
     EST_String name;
     for (k=start_state->pdf_const().item_start();
@@ -2324,7 +2325,8 @@ void EST_Ngrammar::print_freqs(ostream &os,double floor)
 	backoff_representation->print_freqs(os,p_order);
     else
     {
-	int i,j,k;
+	int i,j;
+        EST_Litem *k;
 	EST_IVector window(p_order-1);
 	
 	for (i=0; i < p_num_states; i++)
@@ -2665,7 +2667,7 @@ EST_Ngrammar::backoff_traverse(EST_BackoffNgrammarState *start_state,
     function(start_state,params);
     
     // and recurse down the tree
-    int k;
+    EST_Litem *k;
     double freq;
     EST_String name;
     for (k=start_state->pdf_const().item_start();
@@ -2696,7 +2698,7 @@ EST_Ngrammar::backoff_traverse(EST_BackoffNgrammarState *start_state,
     {
 	// and recurse down the tree if we haven't
 	// reached the level yet
-	int k;
+	EST_Litem *k;
 	double freq;
 	EST_String name;
 	
